@@ -1,5 +1,7 @@
 """Real and complex root isolation and refinement algorithms. """
 
+from __future__ import print_function, division
+
 from sympy.polys.densebasic import (
     dup_LC, dup_TC, dup_degree,
     dup_strip, dup_reverse,
@@ -28,6 +30,9 @@ from sympy.polys.polyerrors import (
     RefinementFailed,
     DomainError)
 
+from sympy.core.compatibility import range
+
+
 def dup_sturm(f, K):
     """
     Computes the Sturm sequence of ``f`` in ``F[x]``.
@@ -41,13 +46,11 @@ def dup_sturm(f, K):
     Examples
     ========
 
-    >>> from sympy.polys.domains import QQ
-    >>> from sympy.polys.rootisolation import dup_sturm
+    >>> from sympy.polys import ring, QQ
+    >>> R, x = ring("x", QQ)
 
-    >>> f = QQ.map([1, -2, 1, -3])
-
-    >>> dup_sturm(f, QQ)
-    [[1/1, -2/1, 1/1, -3/1], [3/1, -4/1, 1/1], [2/9, 25/9], [-2079/4]]
+    >>> R.dup_sturm(x**3 - 2*x**2 + x - 3)
+    [x**3 - 2*x**2 + x - 3, 3*x**2 - 4*x + 1, 2/9*x + 25/9, -2079/4]
 
     References
     ==========
@@ -55,7 +58,7 @@ def dup_sturm(f, K):
     1. [Davenport88]_
 
     """
-    if not (K.has_Field or not K.is_Exact):
+    if not K.is_Field:
         raise DomainError("can't compute Sturm sequence over %s" % K)
 
     f = dup_sqf_part(f, K)
@@ -69,46 +72,63 @@ def dup_sturm(f, K):
     return sturm[:-1]
 
 def dup_root_upper_bound(f, K):
-    """Compute LMQ upper bound for `f`'s positive roots. """
-    n, t, P = len(f), K.one, []
+    """Compute the LMQ upper bound for the positive roots of `f`;
+       LMQ (Local Max Quadratic) was developed by Akritas-Strzebonski-Vigklas.
 
+       Reference:
+       ==========
+       Alkiviadis G. Akritas: "Linear and Quadratic Complexity Bounds on the
+           Values of the Positive Roots of Polynomials"
+           Journal of Universal Computer Science, Vol. 15, No. 3, 523-537, 2009.
+    """
+    n, P = len(f), []
+    t = n * [K.one]
     if dup_LC(f, K) < 0:
         f = dup_neg(f, K)
-
     f = list(reversed(f))
 
-    for i in xrange(0, n):
+    for i in range(0, n):
         if f[i] >= 0:
             continue
 
-        a, Q = K.log(-f[i], 2), []
+        a, QL = K.log(-f[i], 2), []
 
-        for j in xrange(i+1, n):
+        for j in range(i + 1, n):
 
             if f[j] <= 0:
                 continue
 
-            q = t + a - K.log(f[j], 2)
-            Q.append(q // (j - i))
+            q = t[j] + a - K.log(f[j], 2)
+            QL.append([q // (j - i) , j])
 
-            t += 1
-
-        if not Q:
+        if not QL:
             continue
 
-        P.append(min(Q))
+        q = min(QL)
+
+        t[q[1]] = t[q[1]] + 1
+
+        P.append(q[0])
 
     if not P:
         return None
     else:
-        return 2.0**(max(P)+1)
+        return K.get_field()(2)**(max(P) + 1)
 
 def dup_root_lower_bound(f, K):
-    """Compute LMQ lower bound for `f`'s positive roots. """
+    """Compute the LMQ lower bound for the positive roots of `f`;
+       LMQ (Local Max Quadratic) was developed by Akritas-Strzebonski-Vigklas.
+
+       Reference:
+       ==========
+       Alkiviadis G. Akritas: "Linear and Quadratic Complexity Bounds on the
+           Values of the Positive Roots of Polynomials"
+           Journal of Universal Computer Science, Vol. 15, No. 3, 523-537, 2009.
+    """
     bound = dup_root_upper_bound(dup_reverse(f), K)
 
     if bound is not None:
-        return 1.0 / bound
+        return 1/bound
     else:
         return None
 
@@ -159,7 +179,7 @@ def dup_step_refine_real_root(f, M, K, fast=False):
 
     f, g = dup_shift(f, K.one, K), f
 
-    a1, b1, c1, d1 = a, a+b, c, c+d
+    a1, b1, c1, d1 = a, a + b, c, c + d
 
     if not dup_eval(f, K.zero, K):
         return f, (b1, b1, d1, d1)
@@ -174,7 +194,7 @@ def dup_step_refine_real_root(f, M, K, fast=False):
         if not dup_eval(f, K.zero, K):
             f = dup_rshift(f, 1, K)
 
-        a, b, c, d = b, a+b, d, c+d
+        a, b, c, d = b, a + b, d, c + d
 
     return f, (a, b, c, d)
 
@@ -188,10 +208,11 @@ def dup_inner_refine_real_root(f, M, K, eps=None, steps=None, disjoint=None, fas
         a, b, c, d = M
 
     while not c:
-        f, (a, b, c, d) = dup_step_refine_real_root(f, (a, b, c, d), K, fast=fast)
+        f, (a, b, c, d) = dup_step_refine_real_root(f, (a, b, c,
+            d), K, fast=fast)
 
     if eps is not None and steps is not None:
-        for i in xrange(0, steps):
+        for i in range(0, steps):
             if abs(F(a, c) - F(b, d)) >= eps:
                 f, (a, b, c, d) = dup_step_refine_real_root(f, (a, b, c, d), K, fast=fast)
             else:
@@ -202,7 +223,7 @@ def dup_inner_refine_real_root(f, M, K, eps=None, steps=None, disjoint=None, fas
                 f, (a, b, c, d) = dup_step_refine_real_root(f, (a, b, c, d), K, fast=fast)
 
         if steps is not None:
-            for i in xrange(0, steps):
+            for i in range(0, steps):
                 f, (a, b, c, d) = dup_step_refine_real_root(f, (a, b, c, d), K, fast=fast)
 
     if disjoint is not None:
@@ -258,15 +279,25 @@ def dup_refine_real_root(f, s, t, K, eps=None, steps=None, disjoint=None, fast=F
         else:
             disjoint = None
 
-    s, t = dup_outer_refine_real_root(f, s, t, K, eps=eps, steps=steps, disjoint=disjoint, fast=fast)
+    s, t = dup_outer_refine_real_root(
+        f, s, t, K, eps=eps, steps=steps, disjoint=disjoint, fast=fast)
 
     if negative:
         return (-t, -s)
     else:
-        return ( s,  t)
+        return ( s, t)
 
 def dup_inner_isolate_real_roots(f, K, eps=None, fast=False):
-    """Internal function for isolation positive roots up to given precision. """
+    """Internal function for isolation positive roots up to given precision.
+
+       References:
+       ===========
+           1. Alkiviadis G. Akritas and Adam W. Strzebonski: A Comparative Study of Two Real Root
+           Isolation Methods . Nonlinear Analysis: Modelling and Control, Vol. 10, No. 4, 297-304, 2005.
+           2. Alkiviadis G. Akritas, Adam W. Strzebonski and Panagiotis S. Vigklas: Improving the
+           Performance of the Continued Fractions Method Using new Bounds of Positive Roots. Nonlinear
+           Analysis: Modelling and Control, Vol. 13, No. 3, 265-279, 2008.
+    """
     a, b, c, d = K.one, K.zero, K.zero, K.one
 
     k = dup_sign_variations(f, K)
@@ -274,7 +305,8 @@ def dup_inner_isolate_real_roots(f, K, eps=None, fast=False):
     if k == 0:
         return []
     if k == 1:
-        roots = [dup_inner_refine_real_root(f, (a, b, c, d), K, eps=eps, fast=fast, mobius=True)]
+        roots = [dup_inner_refine_real_root(
+            f, (a, b, c, d), K, eps=eps, fast=fast, mobius=True)]
     else:
         roots, stack = [], [(a, b, c, d, f, k)]
 
@@ -305,12 +337,13 @@ def dup_inner_isolate_real_roots(f, K, eps=None, fast=False):
                 if k == 0:
                     continue
                 if k == 1:
-                    roots.append(dup_inner_refine_real_root(f, (a, b, c, d), K, eps=eps, fast=fast, mobius=True))
+                    roots.append(dup_inner_refine_real_root(
+                        f, (a, b, c, d), K, eps=eps, fast=fast, mobius=True))
                     continue
 
             f1 = dup_shift(f, K.one, K)
 
-            a1, b1, c1, d1, r = a, a+b, c, c+d, 0
+            a1, b1, c1, d1, r = a, a + b, c, c + d, 0
 
             if not dup_TC(f1, K):
                 roots.append((f1, (b1, b1, d1, d1)))
@@ -319,7 +352,7 @@ def dup_inner_isolate_real_roots(f, K, eps=None, fast=False):
             k1 = dup_sign_variations(f1, K)
             k2 = k - k1 - r
 
-            a2, b2, c2, d2 = b, a+b, d, c+d
+            a2, b2, c2, d2 = b, a + b, d, c + d
 
             if k2 > 1:
                 f2 = dup_shift(dup_reverse(f), K.one, K)
@@ -346,7 +379,8 @@ def dup_inner_isolate_real_roots(f, K, eps=None, fast=False):
                     f1 = dup_rshift(f1, 1, K)
 
             if k1 == 1:
-                roots.append(dup_inner_refine_real_root(f1, (a1, b1, c1, d1), K, eps=eps, fast=fast, mobius=True))
+                roots.append(dup_inner_refine_real_root(
+                    f1, (a1, b1, c1, d1), K, eps=eps, fast=fast, mobius=True))
             else:
                 stack.append((a1, b1, c1, d1, f1, k1))
 
@@ -360,7 +394,8 @@ def dup_inner_isolate_real_roots(f, K, eps=None, fast=False):
                     f2 = dup_rshift(f2, 1, K)
 
             if k2 == 1:
-                roots.append(dup_inner_refine_real_root(f2, (a2, b2, c2, d2), K, eps=eps, fast=fast, mobius=True))
+                roots.append(dup_inner_refine_real_root(
+                    f2, (a2, b2, c2, d2), K, eps=eps, fast=fast, mobius=True))
             else:
                 stack.append((a2, b2, c2, d2, f2, k2))
 
@@ -453,7 +488,16 @@ def _isolate_zero(f, K, inf, sup, basis=False, sqf=False):
     return [], f
 
 def dup_isolate_real_roots_sqf(f, K, eps=None, inf=None, sup=None, fast=False, blackbox=False):
-    """Isolate real roots of a square-free polynomial using CF approach. """
+    """Isolate real roots of a square-free polynomial using the Vincent-Akritas-Strzebonski (VAS) CF approach.
+
+       References:
+       ===========
+       1. Alkiviadis G. Akritas and Adam W. Strzebonski: A Comparative Study of Two Real Root Isolation Methods.
+       Nonlinear Analysis: Modelling and Control, Vol. 10, No. 4, 297-304, 2005.
+       2. Alkiviadis G. Akritas, Adam W. Strzebonski and Panagiotis S. Vigklas: Improving the Performance
+       of the Continued Fractions Method Using New Bounds of Positive Roots.
+       Nonlinear Analysis: Modelling and Control, Vol. 13, No. 3, 265-279, 2008.
+    """
     if K.is_QQ:
         (_, f), K = dup_clear_denoms(f, K, convert=True), K.get_ring()
     elif not K.is_ZZ:
@@ -475,7 +519,16 @@ def dup_isolate_real_roots_sqf(f, K, eps=None, inf=None, sup=None, fast=False, b
         return [ RealInterval((a, b), f, K) for (a, b) in roots ]
 
 def dup_isolate_real_roots(f, K, eps=None, inf=None, sup=None, basis=False, fast=False):
-    """Isolate real roots using continued fractions approach. """
+    """Isolate real roots using Vincent-Akritas-Strzebonski (VAS) continued fractions approach.
+
+       References:
+       ===========
+       1. Alkiviadis G. Akritas and Adam W. Strzebonski: A Comparative Study of Two Real Root Isolation Methods.
+       Nonlinear Analysis: Modelling and Control, Vol. 10, No. 4, 297-304, 2005.
+       2. Alkiviadis G. Akritas, Adam W. Strzebonski and Panagiotis S. Vigklas: Improving the Performance
+       of the Continued Fractions Method Using New Bounds of Positive Roots.
+       Nonlinear Analysis: Modelling and Control, Vol. 13, No. 3, 265-279, 2008.
+    """
     if K.is_QQ:
         (_, f), K = dup_clear_denoms(f, K, convert=True), K.get_ring()
     elif not K.is_ZZ:
@@ -503,7 +556,16 @@ def dup_isolate_real_roots(f, K, eps=None, inf=None, sup=None, basis=False, fast
     return sorted(I_neg + I_zero + I_pos)
 
 def dup_isolate_real_roots_list(polys, K, eps=None, inf=None, sup=None, strict=False, basis=False, fast=False):
-    """Isolate real roots of a list of square-free polynomial using CF approach. """
+    """Isolate real roots of a list of square-free polynomial using Vincent-Akritas-Strzebonski (VAS) CF approach.
+
+       References:
+       ===========
+       1. Alkiviadis G. Akritas and Adam W. Strzebonski: A Comparative Study of Two Real Root Isolation Methods.
+       Nonlinear Analysis: Modelling and Control, Vol. 10, No. 4, 297-304, 2005.
+       2. Alkiviadis G. Akritas, Adam W. Strzebonski and Panagiotis S. Vigklas: Improving the Performance
+       of the Continued Fractions Method Using New Bounds of Positive Roots.
+       Nonlinear Analysis: Modelling and Control, Vol. 13, No. 3, 265-279, 2008.
+"""
     if K.is_QQ:
         K, F, polys = K.get_ring(), K, polys[:]
 
@@ -571,38 +633,36 @@ def _disjoint_p(M, N, strict=False):
     if not strict:
         return a2*d1 >= c2*b1 or b2*c1 <= d2*a1
     else:
-        return a2*d1 >  c2*b1 or b2*c1 <  d2*a1
+        return a2*d1 > c2*b1 or b2*c1 < d2*a1
 
 def _real_isolate_and_disjoin(factors, K, eps=None, inf=None, sup=None, strict=False, basis=False, fast=False):
     """Isolate real roots of a list of polynomials and disjoin intervals. """
     I_pos, I_neg = [], []
 
     for i, (f, k) in enumerate(factors):
-        for F, M in dup_inner_isolate_positive_roots(f, K,
-                eps=eps, inf=inf, sup=sup, fast=fast, mobius=True):
+        for F, M in dup_inner_isolate_positive_roots(f, K, eps=eps, inf=inf, sup=sup, fast=fast, mobius=True):
             I_pos.append((F, M, k, f))
 
-        for G, N in dup_inner_isolate_negative_roots(f, K,
-                eps=eps, inf=inf, sup=sup, fast=fast, mobius=True):
+        for G, N in dup_inner_isolate_negative_roots(f, K, eps=eps, inf=inf, sup=sup, fast=fast, mobius=True):
             I_neg.append((G, N, k, f))
 
     for i, (f, M, k, F) in enumerate(I_pos):
-        for j, (g, N, m, G) in enumerate(I_pos[i+1:]):
+        for j, (g, N, m, G) in enumerate(I_pos[i + 1:]):
             while not _disjoint_p(M, N, strict=strict):
                 f, M = dup_inner_refine_real_root(f, M, K, steps=1, fast=fast, mobius=True)
                 g, N = dup_inner_refine_real_root(g, N, K, steps=1, fast=fast, mobius=True)
 
-            I_pos[i+j+1] = (g, N, m, G)
+            I_pos[i + j + 1] = (g, N, m, G)
 
         I_pos[i] = (f, M, k, F)
 
     for i, (f, M, k, F) in enumerate(I_neg):
-        for j, (g, N, m, G) in enumerate(I_neg[i+1:]):
+        for j, (g, N, m, G) in enumerate(I_neg[i + 1:]):
             while not _disjoint_p(M, N, strict=strict):
                 f, M = dup_inner_refine_real_root(f, M, K, steps=1, fast=fast, mobius=True)
                 g, N = dup_inner_refine_real_root(g, N, K, steps=1, fast=fast, mobius=True)
 
-            I_neg[i+j+1] = (g, N, m, G)
+            I_neg[i + j + 1] = (g, N, m, G)
 
         I_neg[i] = (f, M, k, F)
 
@@ -630,10 +690,10 @@ def _real_isolate_and_disjoin(factors, K, eps=None, inf=None, sup=None, strict=F
 
     if not basis:
         I_neg = [ ((-v, -u), k) for ((u, v), k, _) in I_neg ]
-        I_pos = [ (( u,  v), k) for ((u, v), k, _) in I_pos ]
+        I_pos = [ (( u, v), k) for ((u, v), k, _) in I_pos ]
     else:
         I_neg = [ ((-v, -u), k, f) for ((u, v), k, f) in I_neg ]
-        I_pos = [ (( u,  v), k, f) for ((u, v), k, f) in I_pos ]
+        I_pos = [ (( u, v), k, f) for ((u, v), k, f) in I_pos ]
 
     return I_neg, I_pos
 
@@ -642,7 +702,7 @@ def dup_count_real_roots(f, K, inf=None, sup=None):
     if dup_degree(f) <= 0:
         return 0
 
-    if not K.has_Field and K.is_Exact:
+    if not K.is_Field:
         R, K = K, K.get_field()
         f = dup_convert(f, R, K)
 
@@ -665,19 +725,25 @@ def dup_count_real_roots(f, K, inf=None, sup=None):
 
     return count
 
-OO = 'OO' # Origin of (re, im) coordinate system
+OO = 'OO'  # Origin of (re, im) coordinate system
 
-Q1 = 'Q1' # Quadrant #1 (++): re > 0 and im > 0
-Q2 = 'Q2' # Quadrant #2 (-+): re < 0 and im > 0
-Q3 = 'Q3' # Quadrant #3 (--): re < 0 and im < 0
-Q4 = 'Q4' # Quadrant #4 (+-): re > 0 and im < 0
+Q1 = 'Q1'  # Quadrant #1 (++): re > 0 and im > 0
+Q2 = 'Q2'  # Quadrant #2 (-+): re < 0 and im > 0
+Q3 = 'Q3'  # Quadrant #3 (--): re < 0 and im < 0
+Q4 = 'Q4'  # Quadrant #4 (+-): re > 0 and im < 0
 
-A1 = 'A1' # Axis #1 (+0): re > 0 and im = 0
-A2 = 'A2' # Axis #2 (0+): re = 0 and im > 0
-A3 = 'A3' # Axis #3 (-0): re < 0 and im = 0
-A4 = 'A4' # Axis #4 (0-): re = 0 and im < 0
+A1 = 'A1'  # Axis #1 (+0): re > 0 and im = 0
+A2 = 'A2'  # Axis #2 (0+): re = 0 and im > 0
+A3 = 'A3'  # Axis #3 (-0): re < 0 and im = 0
+A4 = 'A4'  # Axis #4 (0-): re = 0 and im < 0
 
 _rules_simple = {
+    # Q --> Q (same) => no change
+    (Q1, Q1): 0,
+    (Q2, Q2): 0,
+    (Q3, Q3): 0,
+    (Q4, Q4): 0,
+
     # A -- CCW --> Q => +1/4 (CCW)
     (A1, Q1): 1,
     (A2, Q2): 1,
@@ -740,7 +806,7 @@ _rules_ambiguous = {
     (Q3, OO, A3): -4,
     (Q4, OO, A4): -4,
 
-    # A --  OO --> A => { +1/1 (CCW), -1/1 (CW) }
+    # A --  OO --> A => { +1 (CCW), -1 (CW) }
     (A1, A3): 7,
     (A2, A4): 7,
     (A3, A1): 7,
@@ -751,7 +817,7 @@ _rules_ambiguous = {
     (A3, OO, A1): 7,
     (A4, OO, A2): 7,
 
-    # Q -- DIA --> Q => { +1/1 (CCW), -1/1 (CW) }
+    # Q -- DIA --> Q => { +1 (CCW), -1 (CW) }
     (Q1, Q3): 8,
     (Q2, Q4): 8,
     (Q3, Q1): 8,
@@ -840,13 +906,13 @@ _rules_ambiguous = {
     (Q3, OO, Q2): 16,
     (Q4, OO, Q3): 16,
 
-    # A --> OO --> A => { +2/1 (CCW), 0 (CW) }
+    # A --> OO --> A => { +2 (CCW), 0 (CW) }
     (A1, OO, A1): 17,
     (A2, OO, A2): 17,
     (A3, OO, A3): 17,
     (A4, OO, A4): 17,
 
-    # Q --> OO --> Q => { +2/1 (CCW), 0 (CW) }
+    # Q --> OO --> Q => { +2 (CCW), 0 (CW) }
     (Q1, OO, Q1): 18,
     (Q2, OO, Q2): 18,
     (Q3, OO, Q3): 18,
@@ -854,19 +920,20 @@ _rules_ambiguous = {
 }
 
 _values = {
-     1: [(+1, 4)],
-     2: [(-1, 4)],
-     3: [(+1, 4)],
-     4: [(-1, 4)],
+    0: [( 0, 1)],
+    1: [(+1, 4)],
+    2: [(-1, 4)],
+    3: [(+1, 4)],
+    4: [(-1, 4)],
     -1: [(+9, 4), (+1, 4)],
     -2: [(+7, 4), (-1, 4)],
     -3: [(+9, 4), (+1, 4)],
     -4: [(+7, 4), (-1, 4)],
     +5: [(+1, 2)],
     -5: [(-1, 2)],
-     7: [(+1, 1), (-1, 1)],
-     8: [(+1, 1), (-1, 1)],
-     9: [(+1, 2), (-3, 2)],
+    7: [(+1, 1), (-1, 1)],
+    8: [(+1, 1), (-1, 1)],
+    9: [(+1, 2), (-3, 2)],
     10: [(+3, 2), (-1, 2)],
     11: [(+3, 4), (-5, 4)],
     12: [(+5, 4), (-3, 4)],
@@ -913,7 +980,7 @@ def _intervals_to_quadrants(intervals, f1, f2, s, t, F):
             else:
                 (a, _), _, _ = intervals[1]
 
-                if dup_eval(f2, (s+a)/2, F) > 0:
+                if dup_eval(f2, (s + a)/2, F) > 0:
                     Q.extend([OO, A2])
                     f2_sgn = +1
                 else:
@@ -955,7 +1022,7 @@ def _intervals_to_quadrants(intervals, f1, f2, s, t, F):
             else:
                 (a, _), _, _ = intervals[1]
 
-                if dup_eval(f1, (s+a)/2, F) > 0:
+                if dup_eval(f1, (s + a)/2, F) > 0:
                     Q.extend([OO, A1])
                     f1_sgn = +1
                 else:
@@ -997,8 +1064,8 @@ def _intervals_to_quadrants(intervals, f1, f2, s, t, F):
         else:
             (a, _), _, _ = intervals[1]
 
-            re = dup_eval(f1, (s+a)/2, F)
-            im = dup_eval(f2, (s+a)/2, F)
+            re = dup_eval(f1, (s + a)/2, F)
+            im = dup_eval(f2, (s + a)/2, F)
 
         intervals = intervals[1:]
 
@@ -1097,7 +1164,7 @@ def _traverse_quadrants(Q_L1, Q_L2, Q_L3, Q_L4, exclude=None):
         q1, k = Q[0], 1
 
         while k < len(Q):
-            q2, k = Q[k], k+1
+            q2, k = Q[k], k + 1
 
             if q2 != OO:
                 qq = (q1, q2)
@@ -1109,7 +1176,7 @@ def _traverse_quadrants(Q_L1, Q_L2, Q_L3, Q_L4, exclude=None):
                 else:
                     raise NotImplementedError("2 element rule (inside): " + str(qq))
             else:
-                qq, k = (q1, q2, Q[k]), k+1
+                qq, k = (q1, q2, Q[k]), k + 1
 
                 if qq in _rules_ambiguous:
                     rules.append((_rules_ambiguous[qq], edges[i]))
@@ -1245,7 +1312,7 @@ def _vertical_bisection(N, a, b, I, Q, F1, F2, f1, f2, F):
 
                 if b <= x:
                     I_L1_L.append(((a, b), indices, h))
-                else:
+                if a >= x:
                     I_L1_R.append(((a, b), indices, h))
 
     for I in I_L3:
@@ -1269,18 +1336,18 @@ def _vertical_bisection(N, a, b, I, Q, F1, F2, f1, f2, F):
 
                 if b <= x:
                     I_L3_L.append(((b, a), indices, h))
-                else:
+                if a >= x:
                     I_L3_R.append(((b, a), indices, h))
 
     Q_L1_L = _intervals_to_quadrants(I_L1_L, f1L1F, f2L1F, u, x, F)
-    Q_L2_L = _intervals_to_quadrants(I_L2_L, f1V,   f2V,   v, t, F)
+    Q_L2_L = _intervals_to_quadrants(I_L2_L, f1V, f2V, v, t, F)
     Q_L3_L = _intervals_to_quadrants(I_L3_L, f1L3F, f2L3F, x, u, F)
     Q_L4_L = Q_L4
 
     Q_L1_R = _intervals_to_quadrants(I_L1_R, f1L1F, f2L1F, x, s, F)
     Q_L2_R = Q_L2
     Q_L3_R = _intervals_to_quadrants(I_L3_R, f1L3F, f2L3F, s, x, F)
-    Q_L4_R = _intervals_to_quadrants(I_L4_R, f1V,   f2V,   t, v, F)
+    Q_L4_R = _intervals_to_quadrants(I_L4_R, f1V, f2V, t, v, F)
 
     T_L = _traverse_quadrants(Q_L1_L, Q_L2_L, Q_L3_L, Q_L4_L, exclude=True)
     T_R = _traverse_quadrants(Q_L1_R, Q_L2_R, Q_L3_R, Q_L4_R, exclude=True)
@@ -1351,7 +1418,7 @@ def _horizontal_bisection(N, a, b, I, Q, F1, F2, f1, f2, F):
 
                 if b <= y:
                     I_L2_B.append(((a, b), indices, h))
-                else:
+                if a >= y:
                     I_L2_U.append(((a, b), indices, h))
 
     for I in I_L4:
@@ -1375,15 +1442,15 @@ def _horizontal_bisection(N, a, b, I, Q, F1, F2, f1, f2, F):
 
                 if b <= y:
                     I_L4_B.append(((b, a), indices, h))
-                else:
+                if a >= y:
                     I_L4_U.append(((b, a), indices, h))
 
     Q_L1_B = Q_L1
     Q_L2_B = _intervals_to_quadrants(I_L2_B, f1L2F, f2L2F, v, y, F)
-    Q_L3_B = _intervals_to_quadrants(I_L3_B, f1H,   f2H,   s, u, F)
+    Q_L3_B = _intervals_to_quadrants(I_L3_B, f1H, f2H, s, u, F)
     Q_L4_B = _intervals_to_quadrants(I_L4_B, f1L4F, f2L4F, y, v, F)
 
-    Q_L1_U = _intervals_to_quadrants(I_L1_U, f1H,   f2H,   u, s, F)
+    Q_L1_U = _intervals_to_quadrants(I_L1_U, f1H, f2H, u, s, F)
     Q_L2_U = _intervals_to_quadrants(I_L2_U, f1L2F, f2L2F, y, t, F)
     Q_L3_U = Q_L3
     Q_L4_U = _intervals_to_quadrants(I_L4_U, f1L4F, f2L4F, t, y, F)
@@ -1538,13 +1605,15 @@ def dup_isolate_complex_roots_sqf(f, K, eps=None, inf=None, sup=None, blackbox=F
 
             if N_B >= 1:
                 if N_B == 1 and _rectangle_small_p(a, b, eps):
-                    roots.append(ComplexInterval(a, b, I_B, Q_B, F1_B, F2_B, f1, f2, F))
+                    roots.append(ComplexInterval(
+                        a, b, I_B, Q_B, F1_B, F2_B, f1, f2, F))
                 else:
                     rectangles.append(D_B)
 
             if N_U >= 1:
                 if N_U == 1 and _rectangle_small_p(c, d, eps):
-                    roots.append(ComplexInterval(c, d, I_U, Q_U, F1_U, F2_U, f1, f2, F))
+                    roots.append(ComplexInterval(
+                        c, d, I_U, Q_U, F1_U, F2_U, f1, f2, F))
                 else:
                     rectangles.append(D_U)
 
@@ -1560,8 +1629,9 @@ def dup_isolate_complex_roots_sqf(f, K, eps=None, inf=None, sup=None, blackbox=F
 
 def dup_isolate_all_roots_sqf(f, K, eps=None, inf=None, sup=None, fast=False, blackbox=False):
     """Isolate real and complex roots of a square-free polynomial ``f``. """
-    return (dup_isolate_real_roots_sqf(f, K, eps=eps, inf=inf, sup=sup, fast=fast, blackbox=blackbox),
-            dup_isolate_complex_roots_sqf(f, K, eps=eps, inf=inf, sup=sup, blackbox=blackbox))
+    return (
+        dup_isolate_real_roots_sqf( f, K, eps=eps, inf=inf, sup=sup, fast=fast, blackbox=blackbox),
+        dup_isolate_complex_roots_sqf(f, K, eps=eps, inf=inf, sup=sup, blackbox=blackbox))
 
 def dup_isolate_all_roots(f, K, eps=None, inf=None, sup=None, fast=False):
     """Isolate real and complex roots of a non-square-free polynomial ``f``. """
@@ -1573,14 +1643,15 @@ def dup_isolate_all_roots(f, K, eps=None, inf=None, sup=None, fast=False):
     if len(factors) == 1:
         ((f, k),) = factors
 
-        real_part, complex_part = dup_isolate_all_roots_sqf(f, K, eps=eps, inf=inf, sup=sup, fast=fast)
+        real_part, complex_part = dup_isolate_all_roots_sqf(
+            f, K, eps=eps, inf=inf, sup=sup, fast=fast)
 
         real_part = [ ((a, b), k) for (a, b) in real_part ]
         complex_part = [ ((a, b), k) for (a, b) in complex_part ]
 
         return real_part, complex_part
     else:
-        raise NotImplementedError("only trivial square-free polynomials are supported")
+        raise NotImplementedError( "only trivial square-free polynomials are supported")
 
 class RealInterval(object):
     """A fully qualified representation of a real isolation interval. """
@@ -1617,20 +1688,22 @@ class RealInterval(object):
         a, b, c, d = self.mobius
 
         if not self.neg:
-            return field(a, c)
+            if a*d < b*c:
+                return field(a, c)
+            return field(b, d)
         else:
+            if a*d > b*c:
+                return -field(a, c)
             return -field(b, d)
 
     @property
     def b(self):
         """Return the position of the right end. """
-        field = self.dom.get_field()
-        a, b, c, d = self.mobius
-
-        if not self.neg:
-            return field(b, d)
-        else:
-            return -field(a, c)
+        was = self.neg
+        self.neg = not was
+        rv = -self.a
+        self.neg = was
+        return rv
 
     @property
     def dx(self):
@@ -1658,41 +1731,47 @@ class RealInterval(object):
         if self.mobius is None:
             return self
 
-        f, mobius = dup_inner_refine_real_root(self.f, self.mobius, self.dom, steps=1, mobius=True)
+        f, mobius = dup_inner_refine_real_root(
+            self.f, self.mobius, self.dom, steps=1, mobius=True)
 
         return RealInterval(mobius + (self.neg,), f, self.dom)
 
     def refine_disjoint(self, other):
         """Refine an isolating interval until it is disjoint with another one. """
-        while not self.is_disjoint(other):
-            self, other = self._inner_refine(), other._inner_refine()
+        expr = self
+        while not expr.is_disjoint(other):
+            expr, other = expr._inner_refine(), other._inner_refine()
 
-        return self, other
+        return expr, other
 
     def refine_size(self, dx):
         """Refine an isolating interval until it is of sufficiently small size. """
-        while not (self.dx < dx):
-            self = self._inner_refine()
+        expr = self
+        while not (expr.dx < dx):
+            expr = expr._inner_refine()
 
-        return self
+        return expr
 
     def refine_step(self, steps=1):
         """Perform several steps of real root refinement algorithm. """
-        for _ in xrange(steps):
-            self = self._inner_refine()
+        expr = self
+        for _ in range(steps):
+            expr = expr._inner_refine()
 
-        return self
+        return expr
 
     def refine(self):
         """Perform one step of real root refinement algorithm. """
         return self._inner_refine()
 
 class ComplexInterval(object):
-    """A fully qualified representation of a complex isolation interval. """
+    """A fully qualified representation of a complex isolation interval.
+    The printed form is shown as (x1, y1) x (x2, y2): the southwest x northeast
+    coordinates of the interval's rectangle."""
 
     def __init__(self, a, b, I, Q, F1, F2, f1, f2, dom, conj=False):
         """Initialize new complex interval with complete information. """
-        self.a, self.b = a, b
+        self.a, self.b = a, b  # the southwest and northeast corner: (x1, y1), (x2, y2)
         self.I, self.Q = I, Q
 
         self.f1, self.F1 = f1, F1
@@ -1756,8 +1835,13 @@ class ComplexInterval(object):
 
     def is_disjoint(self, other):
         """Return ``True`` if two isolation intervals are disjoint. """
-        return ((self.conj != other.conj) or ((self.bx <= other.ax or other.bx <= self.ax) or
-                                              (self.by <= other.ay or other.by <= self.ay)))
+        if self.conj != other.conj:
+            return True
+        re_distinct = (self.bx <= other.ax or other.bx <= self.ax)
+        if re_distinct:
+            return True
+        im_distinct = (self.by <= other.ay or other.by <= self.ay)
+        return im_distinct
 
     def _inner_refine(self):
         """Internal one step complex root refinement procedure. """
@@ -1789,27 +1873,29 @@ class ComplexInterval(object):
 
     def refine_disjoint(self, other):
         """Refine an isolating interval until it is disjoint with another one. """
-        while not self.is_disjoint(other):
-            self, other = self._inner_refine(), other._inner_refine()
+        expr = self
+        while not expr.is_disjoint(other):
+            expr, other = expr._inner_refine(), other._inner_refine()
 
-        return self, other
+        return expr, other
 
     def refine_size(self, dx, dy=None):
         """Refine an isolating interval until it is of sufficiently small size. """
         if dy is None:
             dy = dx
+        expr = self
+        while not (expr.dx < dx and expr.dy < dy):
+            expr = expr._inner_refine()
 
-        while not (self.dx < dx and self.dy < dy):
-            self = self._inner_refine()
-
-        return self
+        return expr
 
     def refine_step(self, steps=1):
         """Perform several steps of complex root refinement algorithm. """
-        for _ in xrange(steps):
-            self = self._inner_refine()
+        expr = self
+        for _ in range(steps):
+            expr = expr._inner_refine()
 
-        return self
+        return expr
 
     def refine(self):
         """Perform one step of complex root refinement algorithm. """
