@@ -5,19 +5,19 @@ lambda functions which can be used to calculate numerical values very fast.
 
 from __future__ import print_function, division
 
+from functools import wraps
 import inspect
 import textwrap
-import warnings
 
-from sympy.external import import_module
-from sympy.core.compatibility import exec_, is_sequence, iterable, string_types
+from sympy.core.compatibility import (exec_, is_sequence, iterable,
+    NotIterable, string_types, range, builtins)
 from sympy.utilities.decorator import doctest_depends_on
-from sympy.utilities.exceptions import SymPyDeprecationWarning
 
 # These are the namespaces the lambda functions will use.
 MATH = {}
 MPMATH = {}
 NUMPY = {}
+TENSORFLOW = {}
 SYMPY = {}
 NUMEXPR = {}
 
@@ -28,12 +28,12 @@ NUMEXPR = {}
 MATH_DEFAULT = {}
 MPMATH_DEFAULT = {}
 NUMPY_DEFAULT = {"I": 1j}
+TENSORFLOW_DEFAULT = {}
 SYMPY_DEFAULT = {}
 NUMEXPR_DEFAULT = {}
 
 # Mappings between sympy and other modules function names.
 MATH_TRANSLATIONS = {
-    "Abs": "fabs",
     "ceiling": "ceil",
     "E": "e",
     "ln": "log",
@@ -56,7 +56,7 @@ MPMATH_TRANSLATIONS = {
     #"uppergamma":"upper_gamma",
     "LambertW": "lambertw",
     "MutableDenseMatrix": "matrix",
-    "ImmutableMatrix": "matrix",
+    "ImmutableDenseMatrix": "matrix",
     "conjugate": "conj",
     "dirichlet_eta": "altzeta",
     "Ei": "ei",
@@ -67,7 +67,6 @@ MPMATH_TRANSLATIONS = {
 }
 
 NUMPY_TRANSLATIONS = {
-    "Abs": "abs",
     "acos": "arccos",
     "acosh": "arccosh",
     "arg": "angle",
@@ -80,12 +79,24 @@ NUMPY_TRANSLATIONS = {
     "E": "e",
     "im": "imag",
     "ln": "log",
-    "MutableDenseMatrix": "matrix",
-    "ImmutableMatrix": "matrix",
-    "Max": "amax",
-    "Min": "amin",
+    "Mod": "mod",
     "oo": "inf",
     "re": "real",
+    "SparseMatrix": "array",
+    "ImmutableSparseMatrix": "array",
+    "Matrix": "array",
+    "MutableDenseMatrix": "array",
+    "ImmutableDenseMatrix": "array",
+}
+
+TENSORFLOW_TRANSLATIONS = {
+    "Abs": "abs",
+    "ceiling": "ceil",
+    "im": "imag",
+    "ln": "log",
+    "Mod": "mod",
+    "conjugate": "conj",
+    "re": "real",
 }
 
 NUMEXPR_TRANSLATIONS = {}
@@ -93,8 +104,9 @@ NUMEXPR_TRANSLATIONS = {}
 # Available modules:
 MODULES = {
     "math": (MATH, MATH_DEFAULT, MATH_TRANSLATIONS, ("from math import *",)),
-    "mpmath": (MPMATH, MPMATH_DEFAULT, MPMATH_TRANSLATIONS, ("from sympy.mpmath import *",)),
+    "mpmath": (MPMATH, MPMATH_DEFAULT, MPMATH_TRANSLATIONS, ("from mpmath import *",)),
     "numpy": (NUMPY, NUMPY_DEFAULT, NUMPY_TRANSLATIONS, ("import_module('numpy')",)),
+    "tensorflow": (TENSORFLOW, TENSORFLOW_DEFAULT, TENSORFLOW_TRANSLATIONS, ("import_module('tensorflow')",)),
     "sympy": (SYMPY, SYMPY_DEFAULT, {}, (
         "from sympy.functions import *",
         "from sympy.matrices import *",
@@ -109,10 +121,11 @@ def _import(module, reload="False"):
     Creates a global translation dictionary for module.
 
     The argument module has to be one of the following strings: "math",
-    "mpmath", "numpy", "sympy".
+    "mpmath", "numpy", "sympy", "tensorflow".
     These dictionaries map names of python functions to their equivalent in
     other modules.
     """
+    from sympy.external import import_module
     try:
         namespace, namespace_default, translations, import_commands = MODULES[
             module]
@@ -150,24 +163,40 @@ def _import(module, reload="False"):
     for sympyname, translation in translations.items():
         namespace[sympyname] = namespace[translation]
 
+    # For computing the modulus of a sympy expression we use the builtin abs
+    # function, instead of the previously used fabs function for all
+    # translation modules. This is because the fabs function in the math
+    # module does not accept complex valued arguments. (see issue 9474). The
+    # only exception, where we don't use the builtin abs function is the
+    # mpmath translation module, because mpmath.fabs returns mpf objects in
+    # contrast to abs().
+    if 'Abs' not in namespace:
+        namespace['Abs'] = abs
+
 
 @doctest_depends_on(modules=('numpy'))
 def lambdify(args, expr, modules=None, printer=None, use_imps=True,
-        dummify=True):
+             dummify=True):
     """
     Returns a lambda function for fast calculation of numerical values.
 
-    If not specified differently by the user, SymPy functions are replaced as
-    far as possible by either python-math, numpy (if available) or mpmath
-    functions - exactly in this order. To change this behavior, the "modules"
-    argument can be used. It accepts:
+    If not specified differently by the user, ``modules`` defaults to
+    ``["numpy"]`` if NumPy is installed, and ``["math", "mpmath", "sympy"]``
+    if it isn't, that is, SymPy functions are replaced as far as possible by
+    either ``numpy`` functions if available, and Python's standard library
+    ``math``, or ``mpmath`` functions otherwise. To change this behavior, the
+    "modules" argument can be used. It accepts:
 
-     - the strings "math", "mpmath", "numpy", "numexpr", "sympy"
+     - the strings "math", "mpmath", "numpy", "numexpr", "sympy", "tensorflow"
      - any modules (e.g. math)
      - dictionaries that map names of sympy functions to arbitrary functions
      - lists that contain a mix of the arguments above, with higher priority
        given to entries appearing first.
 
+    .. warning::
+        Note that this function uses ``eval``, and thus shouldn't be used on
+        unsanitized input.
+
     The default behavior is to substitute all arguments in the provided
     expression with dummy symbols. This allows for applied functions (e.g.
     f(t)) to be supplied as arguments. Call the function with dummify=False if
@@ -183,23 +212,19 @@ def lambdify(args, expr, modules=None, printer=None, use_imps=True,
     functions can be found at:
     https://github.com/pydata/numexpr#supported-functions
 
-    Deprecation Warnings
-    ====================
-
     In previous releases ``lambdify`` replaced ``Matrix`` with ``numpy.matrix``
-    by default. As of release 0.7.6 ``numpy.array`` is being transitioned to
-    the default. In release 0.7.7 this transition will be complete. For now, to
-    use the new default behavior you must pass in ``[{'ImmutableMatrix':
-    numpy.array}, 'numpy']`` to the ``modules`` kwarg.
+    by default. As of release 1.0 ``numpy.array`` is the default.
+    To get the old default behavior you must pass in ``[{'ImmutableDenseMatrix':
+    numpy.matrix}, 'numpy']`` to the ``modules`` kwarg.
 
     >>> from sympy import lambdify, Matrix
     >>> from sympy.abc import x, y
     >>> import numpy
-    >>> mat2array = [{'ImmutableMatrix': numpy.array}, 'numpy']
-    >>> f = lambdify((x, y), Matrix([x, y]), modules=mat2array)
+    >>> array2mat = [{'ImmutableDenseMatrix': numpy.matrix}, 'numpy']
+    >>> f = lambdify((x, y), Matrix([x, y]), modules=array2mat)
     >>> f(1, 2)
-    array([[1],
-           [2]])
+    matrix([[1],
+            [2]])
 
     Usage
     =====
@@ -295,6 +320,27 @@ def lambdify(args, expr, modules=None, printer=None, use_imps=True,
 
     ``lambdify`` always prefers ``_imp_`` implementations to implementations
     in other namespaces, unless the ``use_imps`` input parameter is False.
+
+    Usage with Tensorflow module:
+
+    >>> import tensorflow as tf
+    >>> f = Max(x, sin(x))
+    >>> func = lambdify(x, f, 'tensorflow')
+    >>> result = func(tf.constant(1.0))
+    >>> result # a tf.Tensor representing the result of the calculation
+    <tf.Tensor 'Maximum:0' shape=() dtype=float32>
+    >>> sess = tf.Session()
+    >>> sess.run(result) # compute result
+    1.0
+    >>> var = tf.Variable(1.0)
+    >>> sess.run(tf.global_variables_initializer())
+    >>> sess.run(func(var)) # also works for tf.Variable and tf.Placeholder
+    1.0
+    >>> tensor = tf.constant([[1.0, 2.0], [3.0, 4.0]]) # works with any shape tensor
+    >>> sess.run(func(tensor))
+    array([[ 1.,  2.],
+           [ 3.,  4.]], dtype=float32)
+
     """
     from sympy.core.symbol import Symbol
     from sympy.utilities.iterables import flatten
@@ -303,18 +349,16 @@ def lambdify(args, expr, modules=None, printer=None, use_imps=True,
     module_provided = True
     if modules is None:
         module_provided = False
-        # Use either numpy (if available) or python.math where possible.
-        # XXX: This leads to different behaviour on different systems and
-        #      might be the reason for irreproducible errors.
-        modules = ["math", "mpmath", "sympy"]
 
-        #Attempt to import numpy
         try:
             _import("numpy")
         except ImportError:
-            pass
+            # Use either numpy (if available) or python.math where possible.
+            # XXX: This leads to different behaviour on different systems and
+            #      might be the reason for irreproducible errors.
+            modules = ["math", "mpmath", "sympy"]
         else:
-            modules.insert(1, "numpy")
+            modules = ["numpy"]
 
     # Get the needed namespaces.
     namespaces = []
@@ -326,7 +370,7 @@ def lambdify(args, expr, modules=None, printer=None, use_imps=True,
         namespaces.append(modules)
     else:
         # consistency check
-        if 'numexpr' in modules and len(modules) > 1:
+        if _module_present('numexpr', modules) and len(modules) > 1:
             raise TypeError("numexpr must be the only item in 'modules'")
         namespaces += list(modules)
     # fill namespace with first having highest priority
@@ -334,7 +378,6 @@ def lambdify(args, expr, modules=None, printer=None, use_imps=True,
     for m in namespaces[::-1]:
         buf = _get_namespace(m)
         namespace.update(buf)
-    _issue_7853_dep_check(namespaces, namespace, expr)
 
     if hasattr(expr, "atoms"):
         #Try if you can extract symbols from the expression.
@@ -343,10 +386,22 @@ def lambdify(args, expr, modules=None, printer=None, use_imps=True,
         for term in syms:
             namespace.update({str(term): term})
 
-    if 'numexpr' in namespaces and printer is None:
+    if _module_present('mpmath',namespaces) and printer is None:
+        #XXX: This has to be done here because of circular imports
+        from sympy.printing.lambdarepr import MpmathPrinter as printer
+
+    if _module_present('numpy',namespaces) and printer is None:
+        #XXX: This has to be done here because of circular imports
+        from sympy.printing.lambdarepr import NumPyPrinter as printer
+
+    if _module_present('numexpr',namespaces) and printer is None:
         #XXX: This has to be done here because of circular imports
         from sympy.printing.lambdarepr import NumExprPrinter as printer
 
+    if _module_present('tensorflow',namespaces) and printer is None:
+        #XXX: This has to be done here because of circular imports
+        from sympy.printing.lambdarepr import TensorflowPrinter as printer
+
     # Get the names of the args, for creating a docstring
     if not iterable(args):
         args = (args,)
@@ -371,9 +426,21 @@ def lambdify(args, expr, modules=None, printer=None, use_imps=True,
     flat = '__flatten_args__'
 
     if flat in lstr:
-        import itertools
         namespace.update({flat: flatten})
+
+    # Provide lambda expression with builtins, and compatible implementation of range
+    namespace.update({'builtins':builtins, 'range':range})
+
     func = eval(lstr, namespace)
+    # For numpy lambdify, wrap all input arguments in arrays.
+    # This is a fix for gh-11306.
+    if module_provided and _module_present('numpy',namespaces):
+        def array_wrap(funcarg):
+            @wraps(funcarg)
+            def wrapper(*argsx, **kwargsx):
+                return funcarg(*[namespace['asarray'](i) for i in argsx], **kwargsx)
+            return wrapper
+        func = array_wrap(func)
     # Apply the docstring
     sig = "func({0})".format(", ".join(str(i) for i in names))
     sig = textwrap.fill(sig, subsequent_indent=' '*8)
@@ -384,29 +451,14 @@ def lambdify(args, expr, modules=None, printer=None, use_imps=True,
                     "Expression:\n\n{expr}").format(sig=sig, expr=expr_str)
     return func
 
-def _issue_7853_dep_check(namespaces, namespace, expr):
-    """Used for checking things passed into modules kwarg for deprecation
-    issue #7853. This function and the call to it in lambdify should be
-    deleted once the cycle has ended."""
+def _module_present(modname, modlist):
+    if modname in modlist:
+        return True
+    for m in modlist:
+        if hasattr(m, '__name__') and m.__name__ == modname:
+            return True
+    return False
 
-    # If some module changed `ImmutableMatrix` to be something else
-    mat = namespace.get('ImmutableMatrix', False)
-    if not mat or 'numpy' not in namespaces or ('%s.%s' % (mat.__module__,
-            mat.__name__) == 'numpy.matrixlib.defmatrix.matrix'):
-        return
-    dicts = [m for m in namespaces if isinstance(m, dict)]
-    test = lambda expr: hasattr(expr, 'is_Matrix') and expr.is_Matrix
-    if test(expr) and not [d for d in dicts if 'ImmutableMatrix' in d]:
-        SymPyDeprecationWarning(
-                "Currently, `sympy.Matrix` is replaced with `numpy.matrix` if "
-                "the NumPy package is utilized in lambdify. In future versions "
-                "of SymPy (> 0.7.6), we will default to replacing "
-                "`sympy.Matrix` with `numpy.array`. To use the future "
-                "behavior now, supply the kwarg "
-                "`modules=[{'ImmutableMatrix': numpy.array}, 'numpy']`. "
-                "The old behavior can be retained in future versions by "
-                "supplying `modules=[{'ImmutableMatrix': numpy.matrix}, "
-                "'numpy']`.", issue=7853).warn()
 
 def _get_namespace(m):
     """
@@ -495,7 +547,7 @@ def lambdastr(args, expr, printer=None, dummify=False):
 
     # Transform args
     def isiter(l):
-        return iterable(l, exclude=(str, DeferredVector))
+        return iterable(l, exclude=(str, DeferredVector, NotIterable))
 
     if isiter(args) and any(isiter(i) for i in args):
         from sympy.utilities.iterables import flatten
@@ -555,7 +607,8 @@ def _imp_namespace(expr, namespace=None):
        function
 
     Examples
-    --------
+    ========
+
     >>> from sympy.abc import x
     >>> from sympy.utilities.lambdify import implemented_function, _imp_namespace
     >>> from sympy import Function
@@ -624,7 +677,8 @@ def implemented_function(symfunc, implementation):
        function with attached implementation
 
     Examples
-    --------
+    ========
+
     >>> from sympy.abc import x
     >>> from sympy.utilities.lambdify import lambdify, implemented_function
     >>> from sympy import Function
